@@ -21,7 +21,7 @@ use Net::OAI::ListSets;
 use Net::OAI::Record::Header;
 use Net::OAI::Record::OAI_DC;
 
-our $VERSION = '1.0';
+our $VERSION = 'v1.14.1';
 our $DEBUG = 0;
 
 =head1 NAME
@@ -149,7 +149,15 @@ available on the Open Archives Initiative homepage.
 	baseURL => 'http://memory.loc.gov/cgi-bin/oai2_0'
     );
 
-If you would like to fine tune the HTTP client used by Net::OAI::Harvester
+If you want to pull down all the XML files and keep them in a directory, rather
+than having the stored as transient temp files pass in the dumpDir parameter.
+
+    my $harvester = Net::OAI::Harvester->new(
+        baseUrl => 'http://memory.loc.gov/cgi-bin/oai2_0',
+        dumpDir => 'american-memory'
+    );
+
+Also if you would like to fine tune the HTTP client used by Net::OAI::Harvester
 you can pass in a configured object. For example this can be handy if you 
 want to adjust the client timeout:
 
@@ -183,6 +191,14 @@ sub new {
 	$self->userAgent( $ua );
     }
 
+    # set up some stuff if we are dumping xml to a directory
+    if ($normalOpts{ DUMPDIR }) {
+      my $dir = $normalOpts{ DUMPDIR };
+      croak "no such directory '$dir'" unless -d $dir;
+      $self->{ dumpDir } = $dir;
+      $self->{ lastDump } = 0;
+    }
+
     return( $self );
 }
 
@@ -217,7 +233,7 @@ sub identify {
     my $parser = _parser( $error ); 
     debug( "parsing Identify response " .  $identity->file() );
     eval { $parser->parse_uri( $identity->file() ) };
-    if ( $@ ) { _xmlError( $error ); } 
+    if ( $@ ) {_xmlError( $error ); } 
     $identity->{ token } = $token->token() ? $token : undef;
     $identity->{ error } = $error;
     return( $identity );
@@ -570,7 +586,12 @@ Or if you want to know what the current baseURL is
 sub baseURL {
     my ( $self, $url ) = @_;
     if ( $url ) { $self->{ baseURL } = URI->new( $url ); } 
-    return( $self->{ baseURL }->as_string() );
+# The HTTP UserAgent modifies its URI object upon execution,
+# therefore we have to reconstruct: trim the query part ...
+    my $c = $self->{ baseURL }->canonical();
+    if ( $c && ($c =~ /^([^\?]*)\?/) ) {  # $c might be undefined
+        return $1};
+    return $c;
 }
 
 =head2 userAgent()
@@ -596,21 +617,37 @@ sub userAgent {
 sub _get {
     my ($self,$uri) = @_;
     my $ua = $self->{ userAgent };
-    my ( $fh, $file ) = tempfile();
+
+    my ($fh, $file);
+
+    if ( $self->{ dumpDir } ) {
+        my $filePrefix = $self->{lastDump}++;
+        $file = sprintf("%s/%08d.xml", $self->{dumpDir}, $filePrefix);
+        $fh = IO::File->new($file, 'w');
+    }
+
+    else {
+        ( $fh, $file ) = tempfile(UNLINK => 1);
+    }
+
     debug( "fetching ".$uri->as_string() );
-    debug( "writing to tmp file: $file" );
+    debug( "writing to file: $file" );
     my $request = HTTP::Request->new( GET => $uri->as_string() );
     my $response = $ua->request( $request, sub { print $fh shift; }, 4096 );
     close( $fh );
 
     if ( $response->is_error() ) { 
+# HTTP::Request does not provide a file in case of HTTP level errors,
+# therefore we do not return the name of the non-existant file but
+# rather the original HTTP::Response object
         debug( "caught HTTP level error" . $response->message() );
         my $error = Net::OAI::Error->new(
             errorString     => 'HTTP Level Error: ' . $response->message(),
-            errorCode       => $response->code()
+            errorCode       => $response->code(),
+            HTTPError       => $response,
         );
 	return( 
-	    file	    => $file, 
+#	    file	    => $file, 
             error           => $error
 	);
     }
@@ -624,12 +661,15 @@ sub _get {
 sub _parser {
     my $handler = shift;
     my $factory = XML::SAX::ParserFactory->new();
-    return( $factory->parser( Handler => $handler ) );
+    $factory->require_feature(Namespaces);
+    my $parser = $factory->parser( Handler => $handler );
+    debug( "using SAX parser " . ref($parser) . " " . $parser->VERSION );
+    return $parser;
 }
 
 sub _xmlError {
     my $e = shift;
-    debug( "caught xml parsing error" );
+    warn "caught xml parsing error: $@";
     $e->errorString( "XML parsing error: $@" );
     $e->errorCode( 'xmlParseError' );
 }
@@ -649,7 +689,7 @@ sub _verifyMetadataHandler {
 sub debug {
     my $msg = shift; 
     if ( $Net::OAI::Harvester::DEBUG ) { 
-	print STDERR "ockham-harvester: " . localtime() . ": $msg\n";
+	print STDERR "oai-harvester: " . localtime() . ": $msg\n";
     }
 }
 
